@@ -1,5 +1,5 @@
 #
-#  Copyright 2009 10gen, Inc.
+#  Copyright 2009-2013 MongoDB, Inc.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -15,60 +15,53 @@
 #
 
 package MongoDB::GridFS::File;
-our $VERSION = '0.45';
+
 
 # ABSTRACT: A Mongo GridFS file
 
-use Any::Moose;
-use MongoDB::GridFS;
+use version;
+our $VERSION = 'v1.2.2';
+
+use MongoDB::Error;
 use IO::File;
-
-=head1 NAME
-
-MongoDB::GridFS::File - A Mongo GridFS file
-
-=head1 SYNOPSIS
-
-    use MongoDB::GridFS::File;
-
-    my $outfile = IO::File->new("outfile", "w");
-    my $file = $grid->find_one;
-    $file->print($outfile);
-
-=cut
+use Moo;
+use Types::Standard qw(
+    HashRef
+    InstanceOf
+);
+use namespace::clean -except => 'meta';
 
 has _grid => (
     is       => 'ro',
-    isa      => 'MongoDB::GridFS',
+    isa      => InstanceOf['MongoDB::GridFS'],
     required => 1,
 );
 
-=head1 ATTRIBUTES
-
-=head2 info
-
-A hash of info information saved with this file.
-
-=cut
+#pod =attr info
+#pod
+#pod A hash reference of metadata saved with this file.
+#pod
+#pod =cut
 
 has info => (
     is => 'ro',
-    isa => 'HashRef',
+    isa => HashRef,
     required => 1,
 );
 
-=head1 METHODS
 
-=head2 print ($fh, $length?, $offset?)
-
-    $written = $file->print($fh, 50, 200);
-
-Writes the number of bytes specified from the offset specified 
-to the given file handle.  If no C<$length> or C<$offset> are
-given, the entire file is written to C<$fh>.  Returns the number
-of bytes written.
-
-=cut
+#pod =method print
+#pod
+#pod     $written = $file->print($fh);
+#pod     $written = $file->print($fh, $length);
+#pod     $written = $file->print($fh, $length, $offset)
+#pod
+#pod Writes the number of bytes specified from the offset specified 
+#pod to the given file handle.  If no C<$length> or C<$offset> are
+#pod given, the entire file is written to C<$fh>.  Returns the number
+#pod of bytes written.
+#pod
+#pod =cut
 
 sub print {
     my ($self, $fh, $length, $offset) = @_;
@@ -77,10 +70,17 @@ sub print {
     my ($written, $pos) = (0, 0);
     my $start_pos = $fh->getpos();
 
-    $self->_grid->chunks->ensure_index(Tie::IxHash->new(files_id => 1, n => 1));
+    $self->_grid->chunks->ensure_index(Tie::IxHash->new(files_id => 1, n => 1), { safe => 1, unique => 1 });
 
     my $cursor = $self->_grid->chunks->query({"files_id" => $self->info->{"_id"}})->sort({"n" => 1});
 
+    if ( $self->info->{length} && !$cursor->has_next ) {
+        MongoDB::GridFSError->throw(
+            sprintf( "GridFS file corrupt: no chunks found for file ID '%s'",
+                $self->info->{_id} )
+        );
+    }
+
     while ((my $chunk = $cursor->next) && (!$length || $written < $length)) {
         my $len = length $chunk->{'data'};
 
@@ -107,6 +107,7 @@ sub print {
                 $pos += $len-($offset-$pos);
             }
             else {
+                no warnings 'substr';
                 $fh->print(substr($chunk->{'data'}, $offset-$pos, $length));
                 $written += $length;
                 $pos += $length;
@@ -120,17 +121,16 @@ sub print {
     return $written;
 }
 
-=head2 slurp ($length?, $offset?)
-
-    $bytes = $file->slurp(50, 200);
-    $all   = $file->slurp
-
-Return the number of bytes specified from the offset specified
-to the given file handle.  If no C<$length> or C<$offset> are
-given, the entire file is return.
-
-=cut
-
+#pod =method slurp
+#pod
+#pod     $all   = $file->slurp
+#pod     $bytes = $file->slurp($length);
+#pod     $bytes = $file->slurp($length, $offset);
+#pod
+#pod Return the number of bytes specified from the offset specified.  If no
+#pod C<$length> or C<$offset> are given, the entire file is returned.
+#pod
+#pod =cut
 
 sub slurp {
     my ($self,$length,$offset) = @_;
@@ -148,8 +148,114 @@ sub slurp {
     return $bytes;
 }
 
+
 1;
 
-=head1 AUTHOR
+__END__
 
-  Kristina Chodorow <kristina@mongodb.org>
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+MongoDB::GridFS::File - A Mongo GridFS file
+
+=head1 VERSION
+
+version v1.2.2
+
+=head1 SYNOPSIS
+
+    use MongoDB::GridFS::File;
+
+    $outfile = IO::File->new("outfile", "w");
+    $file = $grid->find_one;
+    $file->print($outfile);
+
+=head1 USAGE
+
+=head2 Error handling
+
+Unless otherwise explictly documented, all methods throw exceptions if
+an error occurs.  The error types are documented in L<MongoDB::Error>.
+
+To catch and handle errors, the L<Try::Tiny> and L<Safe::Isa> modules
+are recommended:
+
+    use Try::Tiny;
+    use Safe::Isa; # provides $_isa
+
+    $bytes = try {
+        $file->slurp;
+    }
+    catch {
+        if ( $_->$_isa("MongoDB::TimeoutError" ) {
+            ...
+        }
+        else {
+            ...
+        }
+    };
+
+To retry failures automatically, consider using L<Try::Tiny::Retry>.
+
+=head1 ATTRIBUTES
+
+=head2 info
+
+A hash reference of metadata saved with this file.
+
+=head1 METHODS
+
+=head2 print
+
+    $written = $file->print($fh);
+    $written = $file->print($fh, $length);
+    $written = $file->print($fh, $length, $offset)
+
+Writes the number of bytes specified from the offset specified 
+to the given file handle.  If no C<$length> or C<$offset> are
+given, the entire file is written to C<$fh>.  Returns the number
+of bytes written.
+
+=head2 slurp
+
+    $all   = $file->slurp
+    $bytes = $file->slurp($length);
+    $bytes = $file->slurp($length, $offset);
+
+Return the number of bytes specified from the offset specified.  If no
+C<$length> or C<$offset> are given, the entire file is returned.
+
+=head1 AUTHORS
+
+=over 4
+
+=item *
+
+David Golden <david@mongodb.com>
+
+=item *
+
+Mike Friedman <friedo@friedo.com>
+
+=item *
+
+Kristina Chodorow <k.chodorow@gmail.com>
+
+=item *
+
+Florian Ragwitz <rafl@debian.org>
+
+=back
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is Copyright (c) 2016 by MongoDB, Inc..
+
+This is free software, licensed under:
+
+  The Apache License, Version 2.0, January 2004
+
+=cut
